
//  "Ántrax CPC Game": An Amstrad CPC Game
//  Copyright (C) 2015  Ántrax CPC Team
//
//  This file is part of "Ántrax CPC Game".
//
//  "Ántrax CPC Game" is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  "Ántrax CPC Game" is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "enemy.h"

#include "arktro.h"
#include "shifter.h"


TEnemy enemies[MAX_ENEMIES];


/*
 * Crea un jugador aleatorio en la estructura TEnemy pasada por parámetro.
 */
void createRandomEnemy(TEnemy *e)
{
    // Tipo aleatorio. El + 1 para evitar que sea TYPE_NONE.
    int type = cpct_getRandomUniform_u8_f(frame) % N_TYPES + 1;
    switch (type)
    {
        case TYPE_ARKTRO:
            initArktro(e);
            break;
        case TYPE_SPANIARD:
            //initSpaniard(e);
            break;
        case TYPE_SHIFTER:
             initShifter(e);
    }
}

/*
 * Modifica e->x y e->y para que se acerque lo más posible a e->ox y e->oy.
 * Se moverá usando la velocidad del enemigo (e->sx y e->sy).
 */
void moveEnemyToOXOY(TEnemy *e)
{
    if (e->x < e->ox)
    {
        e->x += e->sx;
    }
    else if (e->x > e->ox)
    {
        e->x -= e->sx;
    }

    if (e->y < e->oy)
    {
        e->y += e->sy;
    }
    else if (e->y > e->oy)
    {
        e->y -= e->sy;
    }
}

/*
 * Planea el siguiente movimiento de un enemigo (da valor a e->ox y a e->oy)
 * en función de su objetivo a largo plazo (parámetros ox, oy).
 *
 * Ej: si un enemigo quiere moverse hacia el jugador, los parámetros ox y oy
 * deben de ser la posición del jugador. Tras acabar la función, la estructura
 * del enemigo tendrá el valor de cual es la siguiente casilla a la que debe
 * moverse guardado en e->ox y en e->oy.
 */
void planMoveTo(TEnemy *e, i16 ox, i16 oy)
{
    i16 x = e->x + 3, y = e->y + 3;
    u8 result, minResult = INFINITE_U8;
    // Variando eje x
    // --------------
    // Expandir derecha
    result = manhattanXY((e->x+8)/8, e->y/8, ox/8, oy/8);
    //player.score = result; gameOver = -1;
    if (result < minResult)
    {
        e->ox = e->x + 8;
        e->oy = e->y;
        minResult = result;
    }
    // Expandir izquierda
    result = manhattanXY((e->x-8)/8, e->y/8, ox/8, oy/8);
    if (result < minResult)
    {
        e->ox = e->x - 8;
        e->oy = e->y;
        minResult = result;
    }
    // Variando eje y
    // --------------
    // Expandiendo arriba
    result = manhattanXY(e->x/8, (e->y-8)/8, ox/8, oy/8);
    if (result < minResult)
    {
        e->ox = e->x;
        e->oy = e->y - 8;
        minResult = result;
    }
    // Expandiendo abajo
    result = manhattanXY(e->x/8, (e->y+8)/8, ox/8, oy/8);
    if (result < minResult)
    {
        e->ox = e->x;
        e->oy = e->y + 8;
        minResult = result;
    }
}

void initRandomEnemies(u8 nEnemies)
{
    u8 i;

    for (i = 0; i < MAX_ENEMIES; ++i)
    {
        if (i < nEnemies)
        {
            createRandomEnemy(&enemies[i]);
        }
        else
        {
            enemies[i].type = TYPE_NONE;
        }
    }
}
