
//  "Ántrax CPC Game": An Amstrad CPC Game
//  Copyright (C) 2015  Ántrax CPC Team
//
//  This file is part of "Ántrax CPC Game".
//
//  "Ántrax CPC Game" is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  "Ántrax CPC Game" is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <stdio.h>
#include <stdlib.h>

#include "gun.h"
#include "enemy.h"
#include "masfly.h"
#include "plutonium.h"
#include "flux.h"
#include "medpack.h"
#include "../utils.h"
#include "../scenes/game.h"


const u8 g_gun[32] = { 
	0xB0, 0xF0, 0xF0, 0xF0, 
	0x30, 0x30, 0x30, 0x30, 
	0x30, 0x30, 0x30, 0x30, 
	0xB0, 0x70, 0x70, 0xF0, 
	0xB0, 0x30, 0x70, 0xF0, 
	0xB0, 0x70, 0xF0, 0xF0, 
	0xB0, 0x70, 0xF0, 0xF0, 
	0xB0, 0x70, 0xF0, 0xF0
};

const u8 g_bullet[8] = { 
	0xB0, 0x70, 
	0x30, 0x30, 
	0x30, 0x30, 
	0xB0, 0x70
};


TBullet bullet1;
TBullet bullet2;

u8 bulletSpeed;


// 0 if the gun is in the left map, 1 if it is in the right map.
u8 gunDimension = 1;
u8 bulletSpeed = BULLET_SPEED;
u16 timeChanged;


/*
 * Pinta la pistola debajo de un mapa sea cual sea la
 * última pulsación (ideal para pintar la interfaz al
 * inicio de cada nivel).
 */
void paintGun()
{
	timeChanged = 0;
	frame += FRAMES_DELAY_TOGGLE;
    toggleGun();
    frame -= FRAMES_DELAY_TOGGLE;
}

void initBullet(TBullet *b)
{
	b->sx = b->sy = 0;
	b->map = 2;
	b->damage = BULLET_DAMAGE;
	b->exists = 0;
}


void createBullet(TBullet *const b, const u8 orientation, const u8 dimension, const u8 x, const u8 y, u16 *const timeChanged)
{
	// Espaciamos en el tiempo los disparos.
	if (frame - *timeChanged < FRAMES_DELAY_SHOT)
	{
		return;
	}
	*timeChanged = frame;

	//b->map = gunDimension;
	b->map = dimension;
	
	//b->x = player.x[gunDimension];
	b->x = x;
	
	//b->y = player.y[gunDimension];
	b->y = y;
	
	//b->orientation = player.orientation;
	b->orientation = orientation;
	
	b->exists = 1;

	switch (orientation)
	{
		case LEFT:
			b->sx = -BULLET_SPEED;
			b->y += 2;
			break;
		case RIGHT:
			b->sx = BULLET_SPEED;
			b->x += 4;
			b->y += 2;
			break;
		case UP:
			b->sy = -BULLET_SPEED;
			b->x += 2;
			break;
		case DOWN:
			b->sy = BULLET_SPEED;
			b->x += 2;
			b->y += 4;
			break;
	}

	/*if (b->y - 2 + 8 > 169 || b->y < 0 || b->x < 0 || b->x > 160 - 4)
		b->map = 2;
	else*/
		b->pvideo = cpct_getScreenPtr(SCR_VMEM, b->x / 2, b->y);
}

void updateBullet(TBullet *b)
{
	u8 destroy = 0;
	cpct_drawSolidBox(b->pvideo, 0xf0, 4 / 2, 4);

	switch (b->orientation)
	{
		case LEFT:
			if (!collision(b->x - BULLET_SPEED, b->y) &&
				!collision(b->x - BULLET_SPEED, b->y + 2))
			{
				b->x -= BULLET_SPEED;
			}
			else
			{
				b->x -= BULLET_SPEED;
				destroy = 1;
			}
			break;
		case RIGHT:
			if (!collision(b->x + 4 + BULLET_SPEED, b->y) &&
				!collision(b->x + 4 + BULLET_SPEED, b->y + 2))
			{
				b->x += BULLET_SPEED;
			}
			else
			{
				b->x += BULLET_SPEED;
				destroy = 1;
			}
			break;
		case UP:
			if (!collision(b->x, b->y - BULLET_SPEED) &&
				!collision(b->x + 2, b->y - BULLET_SPEED))
			{
				b->y -= BULLET_SPEED;
			}
			else
			{
				b->y -= BULLET_SPEED; 
				destroy = 1;
			}
			break;
		case DOWN:
			if (!collision(b->x, b->y + 3 + BULLET_SPEED) &&
				!collision(b->x + 2, b->y + 3 + BULLET_SPEED))
			{
				b->y += BULLET_SPEED;
			}
			else
			{
				b->y += BULLET_SPEED;
				destroy = 1;
			}
			break;
	}
	// Para evitar que no se destruya la bala atravesando paredes estrechas.
	if (collision(b->x + 2, b->y + 1) || collision(b->x + 2, b->y + 2))
	{
		destroy = 1;
	}

	
	if (destroy)
	{
		if (b->y - 2 < 0)
		{
			b->y = 2;
		}
		else if (b->y - 2 > 161)
		{
			b->y = 162;
		}

		if (b->map == 0)
		{
			if (b->x - 2 < 0)
			{
				b->x = 2;
			}
			else if (b->x - 2 > 71)
			{
				b->x = 72;
			}
		}
		else if (b->map == 1)
		{
			if (b->x - 2 < 84)
			{
				b->x = 84;
			}
			else if (b->x - 2 > 153)
			{
				b->x = 153;
			}
		}

		b->pvideo = cpct_getScreenPtr(SCR_VMEM, (b->x - 2) / 2, b->y - 2);
		cpct_drawSolidBox(b->pvideo, 0xf0, 8 / 2, 8);
		player.score++;
		
		if (plutonium.life &&
			!plutonium.catched &&
			abs(b->x - plutonium.x + 2) <= 10 &&
			abs(b->y - plutonium.y + 2) <= 10)
		{
			plutonium.life -= b->damage;
			if (plutonium.life <= 0)
			{
				plutonium.life = 0;
				gameOver = -1;
			}
			else
				// Repintamos plutonio para evitar que las balas lo borren.
				cpct_drawSprite(g_plutonium, plutonium.pvideo, TILE_WIDTH / 2, TILE_HEIGHT);
		}
		else if (flux.life &&
				!flux.catched &&
				abs(b->x - flux.x + 2) <= 10 &&
				abs(b->y - flux.y + 2) <= 10)
		{
			flux.life -= b->damage;
			if (flux.life <= 0)
			{
				flux.life = 0;
				gameOver = -1;
			}
			else
				// Repintamos condensador para evitar que las balas lo borren.
				cpct_drawSprite(g_flux, flux.pvideo, TILE_WIDTH / 2, TILE_HEIGHT);
		}
		else if (medpack.life &&
				!medpack.catched &&
				abs(b->x - medpack.x + 2) <= 10 &&
				abs(b->y - medpack.y + 2) <= 10)
		{
			medpack.life -= b->damage;
			if (medpack.life <= 0)
			{
				medpack.life = 0;
				medpack.catched = 2;
				catchedMedPack();
			}
			else
				// Repintamos condensador para evitar que las balas lo borren.
				cpct_drawSprite(g_medpack, medpack.pvideo, TILE_WIDTH / 2, TILE_HEIGHT);
		}
		else
		{
			u8 i;
			for (i = 0; i < MAX_ENEMIES; ++i)
			{
				if (enemies[i].type &&
					abs(b->x - enemies[i].x + 4) <= 10 &&
					abs(b->y - enemies[i].y + 4) <= 10)
				{
					enemies[i].life -= b->damage;
					if (enemies[i].life <= 0)
					{
						enemies[i].type = TYPE_NONE;
						// TODO
						//player.score += SCORE_KILL_ENEMY;
						// hasta entonces..
						player.score++;
						// Borramos enemigo.
						cpct_drawSolidBox(enemies[i].pvideo, 0xf0, TILE_WIDTH / 2, TILE_HEIGHT);
					}
				}
			}
		}

		b->map = 2;
		b->exists = 0;
	}
	else
	{
		b->pvideo = cpct_getScreenPtr(SCR_VMEM, b->x / 2, b->y);
		cpct_drawSprite(g_bullet, b->pvideo, 4 / 2, 4);
	}
}


/*
 * Cambia el arma de mapa. El mapa en el que se encuentra el arma es
 * indicado por la variable gunDimension: 0 izquierda, 1 derecha.
 */
void toggleGun()
{
	if (frame - timeChanged >= FRAMES_DELAY_TOGGLE)
	{
		timeChanged = frame;
		gunDimension = (gunDimension + 1) % 2;

		if (gunDimension)
		{
			cpct_drawSolidBox(VMEM_RIGHT_GUN, 0xf0, TILE_WIDTH / PIXELS_PER_BYTE, TILE_HEIGHT);
			cpct_drawSprite(g_gun, VMEM_LEFT_GUN, TILE_WIDTH / PIXELS_PER_BYTE, TILE_HEIGHT);
		}
		else
		{
			cpct_drawSolidBox(VMEM_LEFT_GUN, 0xf0, TILE_WIDTH / PIXELS_PER_BYTE, TILE_HEIGHT);
			cpct_drawSprite(g_gun, VMEM_RIGHT_GUN, TILE_WIDTH / PIXELS_PER_BYTE, TILE_HEIGHT);
		}
	}
}
