

// Data created with Img2CPC - (c) Retroworks - 2007-2015


//  "Ántrax CPC Game": An Amstrad CPC Game
//  Copyright (C) 2015  Ántrax CPC Team
//
//  This file is part of "Ántrax CPC Game".
//
//  "Ántrax CPC Game" is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  "Ántrax CPC Game" is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <cpctelera.h>

#include "masfly.h"
#include "../utils.h"
#include "../scenes/game.h"

#define FRAMES_DELAY_REMOVE_LIFE 25


TPlayer player;

const u8 sx = 2;
const u8 sy = 2;

u16 timeRemoveLife;


void initPlayer()
{
	player.x[0] = 7 * 8; player.x[1] = 15 * 8;
	player.y[0] = 5 * 8; player.y[1] = 2 * 8;
    
    player.spr = &tile_masfly_d_s;
   	player.frameSet = tileset_masfly_down;
   	player.framesInSet = PLAYER_VERT_ANIM;
	player.frameCounter = 0;
	player.orientation = DOWN;
	player.timeShot = 0;
	DrawAllLife();
    	
    // Left sprite pointer.
    player.players[0] = cpct_getScreenPtr(SCR_VMEM, player.x[0] / 2, player.y[0]);
    // Right sprite pointer.
	player.players[1] = cpct_getScreenPtr(SCR_VMEM, player.x[1] / 2, player.y[1]);

	timeRemoveLife = 0;
}

void DrawAllLife()
{
	u8 i = 1;
    cpct_drawSprite(tile_masfly_d_s, LIFE_VMEM, TILE_WIDTH / 2, TILE_HEIGHT);

    for(i ; i <= player.life; i++)
    {
    	cpct_drawSolidBox(LIFE_VMEM+2+3*i, 0xcc,2, 8);
    }
}

void putLife(u8 lifes)
{
	u8 i = player.life+1;

	if(player.life+lifes<PLAYER_LIFE)
	{
		player.life+=lifes;
		for(i ; i < player.life+1  ; i++)
		{
			cpct_drawSolidBox(LIFE_VMEM+2+3*i,0xcc,2, 8);
		}
	}
	else
	{
		player.life=PLAYER_LIFE;
		DrawAllLife();
	}
}

void removeLife(u8 lifes)
{
	u8 i = player.life;

	// Espaciamos la quitada de vida para darle un respiro al jugador en caso de daño.
	if (frame - timeRemoveLife < FRAMES_DELAY_REMOVE_LIFE)
	{
		return;
	}
	timeRemoveLife = frame;

	if (player.life - lifes < 0)
	{
		player.life = 0;
	}
	else
	{
		player.life -= lifes;
	}

	for (; i > player.life ; i--)
	{
		cpct_drawSolidBox(LIFE_VMEM + 2 + 3 * i, 0xf0, 2, 8);
	}
}


const u8* const tileset_masfly_up[3] = { 
	tile_masfly_u_s,
	tile_masfly_u_m1,
	tile_masfly_u_m2
};

const u8* const tileset_masfly_down[3] = { 
	tile_masfly_d_s,
	tile_masfly_d_m1,
	tile_masfly_d_m2
};

const u8* const tileset_masfly_left[2] = { 
	tile_masfly_l_s,
	tile_masfly_l_m
};


const u8* const tileset_masfly_right[2] = { 
	tile_masfly_r_s,
	tile_masfly_r_m
};


// Right stop.
const u8 tile_masfly_r_s[32] = {
	0xf0, 0xcc, 0xd8, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xf0,
	0xf0, 0xcf, 0xe5, 0xf0,
	0xf0, 0xcf, 0xda, 0xf0,
	0xe0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xcc, 0xd8, 0xf0,
	0xf0, 0xd0, 0xd0, 0xf0
};

// Right moving.
const u8 tile_masfly_r_m[32] = {
	0xf0, 0xcc, 0xd8, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xf0,
	0xf0, 0xcf, 0xe5, 0xf0,
	0xf0, 0xcf, 0xda, 0xf0,
	0xe0, 0xc0, 0xc0, 0xf0,
	0xf0, 0xcc, 0xd8, 0xf0,
	0xf0, 0xe0, 0xf0, 0xf0
};

// Up stop.
const u8 tile_masfly_u_s[32] = {
	0xf0, 0xcc, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xda,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xe0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xcc, 0xcc, 0xf0,
	0xf0, 0xd0, 0xe0, 0xf0
};

// Up moving 1.
const u8 tile_masfly_u_m1[32] = {
	0xf0, 0xcc, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xda,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xe0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xc4, 0xcc, 0xf0,
	0xf0, 0xf0, 0xe0, 0xf0
};

// Up moving 2.
const u8 tile_masfly_u_m2[32] = {
	0xf0, 0xcc, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xda,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xe0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xcc, 0xc8, 0xf0,
	0xf0, 0xd0, 0xf0, 0xf0
};

// Dead right.
const u8 tile_masfly_d_r[32] = {
	0xf0, 0xcc, 0xd8, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xf0,
	0xf0, 0xcf, 0xcd, 0xf0,
	0xf0, 0xcf, 0xda, 0xf0,
	0xf0, 0xc0, 0xc0, 0xf0,
	0xf0, 0xcc, 0xd8, 0xf0,
	0xf0, 0xe0, 0xe0, 0xf0
};

// Left stop.
const u8 tile_masfly_l_s[32] = {
	0xf0, 0xe4, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xf0, 0xcf, 0xcf, 0xda,
	0xf0, 0xda, 0xcf, 0xf0,
	0xf0, 0xe5, 0xcf, 0xf0,
	0xe0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xe4, 0xcc, 0xf0,
	0xf0, 0xe0, 0xe0, 0xf0
};

// Left moving.
const u8 tile_masfly_l_m[32] = {
	0xf0, 0xe4, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xf0, 0xcf, 0xcf, 0xda,
	0xf0, 0xda, 0xcf, 0xf0,
	0xf0, 0xe5, 0xcf, 0xf0,
	0xf0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xe4, 0xcc, 0xf0,
	0xf0, 0xf0, 0xd0, 0xf0
};

// Down stop.
const u8 tile_masfly_d_s[32] = {
	0xf0, 0xcc, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xda,
	0xe5, 0xe5, 0xda, 0xda,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xe0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xcc, 0xcc, 0xf0,
	0xf0, 0xd0, 0xe0, 0xf0
};

// Down moving 1.
const u8 tile_masfly_d_m1[32] = {
	0xf0, 0xcc, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xda,
	0xe5, 0xe5, 0xda, 0xda,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xe0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xc4, 0xcc, 0xf0,
	0xf0, 0xf0, 0xe0, 0xf0
};

// Down moving 2.
const u8 tile_masfly_d_m2[32] = {
	0xf0, 0xcc, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xe5, 0xcf, 0xcf, 0xda,
	0xe5, 0xe5, 0xda, 0xda,
	0xf0, 0xcf, 0xcf, 0xf0,
	0xe0, 0xc0, 0xc0, 0xd0,
	0xf0, 0xcc, 0xc8, 0xf0,
	0xf0, 0xd0, 0xf0, 0xf0
};

// Dead left.
// FIXME: Remove al least one dead image to save memory.
const u8 tile_masfly_d_l[32] = {
	0xf0, 0xe4, 0xcc, 0xf0,
	0xe4, 0xcc, 0xcc, 0xd8,
	0xf0, 0xcf, 0xcf, 0xda,
	0xf0, 0xce, 0xcf, 0xf0,
	0xf0, 0xe5, 0xcf, 0xf0,
	0xf0, 0xc0, 0xc0, 0xf0,
	0xf0, 0xe4, 0xcc, 0xf0,
	0xf0, 0xd0, 0xd0, 0xf0
};

