
//  "Ántrax CPC Game": An Amstrad CPC Game
//  Copyright (C) 2015  Ántrax CPC Team
//
//  This file is part of "Ántrax CPC Game".
//
//  "Ántrax CPC Game" is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  "Ántrax CPC Game" is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <cpctelera.h>

#include "scenes.h"
#include "game.h"
#include "../entities/masfly.h"
#include "../entities/gun.h"
#include "../entities/plutonium.h"
#include "../entities/flux.h"
#include "../entities/medpack.h"
#include "../entities/arktro.h"
#include "../entities/map.h"
#include "../entities/spaniard.h"



#define TIME_STR_POS (u8*) 0xFF31
#define TIME_NUM_POS (u8*) 0xFF58
#define	PLUTONIUM_FLUX 1
#define TIME_PLUTONIUM 1


// Current frame number from the start of the game.
u16 frame = 0;

// Default level time.
u16 levelTime;

i8 level;

i8 gameOver;


void drawInterface()
{
    u8* pvideo;
    u8 i = 1;

    u8 color = cpct_px2byteM0(10, 10);

    // Split screen.
	pvideo = cpct_getScreenPtr(SCR_VMEM, 79 / 2, 0);
    cpct_drawSolidBox(pvideo, color, 4 / 2, MAP_HEIGHT * TILE_HEIGHT);

    // Info panel.
    // Left.
    pvideo = cpct_getScreenPtr(SCR_VMEM, 0, MAP_HEIGHT * TILE_HEIGHT);
    cpct_drawSolidBox(pvideo, color, 2 / 2, 4 * 8);
    // Right.
    pvideo = cpct_getScreenPtr(SCR_VMEM, 160 / 2 - 1, MAP_HEIGHT * 8);
    cpct_drawSolidBox(pvideo, color, 2 / 2, 4 * 8);
    // Up.
    pvideo = cpct_getScreenPtr(SCR_VMEM, 0, MAP_HEIGHT * 8);
    cpct_memset(pvideo, color, 80);
    // Down.
    pvideo = cpct_getScreenPtr(SCR_VMEM, 0, 199);
    cpct_memset(pvideo, color, 80);

    // TIME info
    cpct_drawStringM0("Tiempo: ", TIME_STR_POS, 1, 5);

    // Gun
    cpct_drawSolidBox(VMEM_LEFT_GUN - 1, color, 10 / 2, 10);
    cpct_drawSolidBox(VMEM_RIGHT_GUN, color, 10 / 2, 10); 
    paintGun();

    // Bag
    cpct_drawSolidBox(VMEM_PLUTONIUM - 1, color, 12 / 2, 10);
    cpct_drawSolidBox(VMEM_PLUTONIUM, 0xf0, TILE_WIDTH / PIXELS_PER_BYTE, TILE_HEIGHT);




}

void showGameOver()
{
	u8* pvideoAux;
	cpct_clearScreen(0xf0);
	cpct_setBorder(HW_RED);

	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 6 * 8 / 2, 8 * 8);
	cpct_drawStringM0("Nivel ", pvideoAux, 1, 5);
	cpct_drawCharM0(pvideoAux + 6*4, 1, 5, ((level / 10) % 10) + '0');
	cpct_drawCharM0(pvideoAux + 7*4, 1, 5, (level % 10) + '0');

	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 3 * 8 / 2, 10 * 8);
	cpct_drawStringM0("Fin del juego", pvideoAux, 1, 5);

	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 7 * 8 / 2, 12 * 8);
	cpct_drawCharM0(pvideoAux     , 1, 5, '0');
	cpct_drawCharM0(pvideoAux +  4, 1, 5, ((player.score / 10000) % 10) + '0');
	cpct_drawCharM0(pvideoAux +  8, 1, 5, ((player.score / 1000) % 10) + '0');
	cpct_drawCharM0(pvideoAux + 12, 1, 5, ((player.score / 100) % 10) + '0');
	cpct_drawCharM0(pvideoAux + 16, 1, 5, ((player.score / 10) % 10) + '0');
	cpct_drawCharM0(pvideoAux + 20, 1, 5, (player.score % 10) + '0');

	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 4 * 8 / 2, 16 * 8);
	cpct_drawSprite(tile_masfly_d_l, pvideoAux, TILE_WIDTH / 2, TILE_HEIGHT);
	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 15 * 8 / 2, 16 * 8);
	cpct_drawSprite(tile_masfly_d_r, pvideoAux, TILE_WIDTH / 2, TILE_HEIGHT);

	while (--pvideoAux);

	releaseAndPushKey();
}

void showYouWin()
{
	u8* pvideoAux;
	cpct_clearScreen(0xf0);
	cpct_setBorder(HW_GREEN);

	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 6 * 8 / 2, 8 * 8);
	cpct_drawStringM0("Ganaste", pvideoAux, 1, 5);

	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 3 * 8 / 2, 10 * 8);
	cpct_drawStringM0("Fin del juego", pvideoAux, 1, 5);

	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 7 * 8 / 2, 12 * 8);
	cpct_drawCharM0(pvideoAux     , 1, 5, '0');
	cpct_drawCharM0(pvideoAux +  4, 1, 5, ((player.score / 10000) % 10) + '0');
	cpct_drawCharM0(pvideoAux +  8, 1, 5, ((player.score / 1000) % 10) + '0');
	cpct_drawCharM0(pvideoAux + 12, 1, 5, ((player.score / 100) % 10) + '0');
	cpct_drawCharM0(pvideoAux + 16, 1, 5, ((player.score / 10) % 10) + '0');
	cpct_drawCharM0(pvideoAux + 20, 1, 5, (player.score % 10) + '0');

	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 4 * 8 / 2, 16 * 8);
	cpct_drawSprite(tile_masfly_d_m1, pvideoAux, TILE_WIDTH / 2, TILE_HEIGHT);
	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 15 * 8 / 2, 16 * 8);
	cpct_drawSprite(tile_masfly_d_m2, pvideoAux, TILE_WIDTH / 2, TILE_HEIGHT);

	while (--pvideoAux);

	releaseAndPushKey();
}

u8 playLevel()
{
	u8* pvideoAux;
	u8  UraAux=0,UraTime=0;

	frame = 0;
	levelTime = 99 + level / 10 * 100;

	
	cpct_clearScreen(0xf0);


	pvideoAux = cpct_getScreenPtr(SCR_VMEM, 6 * 8 / 2, 8 * 8);
	cpct_drawStringM0("Nivel ", pvideoAux, 1, 5);
	cpct_drawCharM0(pvideoAux + 6*4, 1, 5, ((level / 10) % 10) + '0');
	cpct_drawCharM0(pvideoAux + 7*4, 1, 5, (level % 10) + '0');

	while (--pvideoAux);

	/*cpct_setRandomSeedUniform_u8(frame*frame*frame);
	generateMap(map_left);
	frame++;
	generateMap(map_right);
	frame--;*/
	generateMap(map_left);
	generateMap(map_right);
	cpct_clearScreen(0xf0);
	drawMap(MAP_LEFT_VMEM, map_left);
	drawMap(MAP_RIGHT_VMEM, map_right);
    drawInterface();

    initPlayer();
	initPlutonium();
	initFlux();
	initRandomEnemies((level == 1) ? 1 : 2 + level / 10);
    initBullet(&bullet1);
    initBullet(&bullet2);

    if (level == 2 || level >= 4)
    {
    	initMedPack(); //TODO
    }

	// GAME LOOP.
	gameOver = 0;
	while(!gameOver)
	{
		//cpct_waitVSYNC();
		cpct_count2VSYNC();

		cpct_scanKeyboard_f();

		if (frame % 2 == 1) // Odd frames
		{

			if (cpct_isKeyPressed(Key_CursorUp))
			{
				player.frameSet = tileset_masfly_up;
				player.framesInSet = PLAYER_VERT_ANIM;
				player.spr = player.frameSet[player.frameCounter % player.framesInSet];
				player.frameCounter++;
				player.orientation = UP;
				
	            if (!collisionH(player.x[0], player.y[0] - sy))
	            {
	            	player.y[0] -= sy;
	            }
	            if (!collisionH(player.x[1], player.y[1] - sy))
	            {
	            	player.y[1] -= sy;
	            }
	        }
			else if (cpct_isKeyPressed(Key_CursorDown))
			{
				player.frameSet = tileset_masfly_down;
				player.framesInSet = PLAYER_VERT_ANIM;
				player.spr = player.frameSet[player.frameCounter % player.framesInSet];
				player.frameCounter++;
				player.orientation = DOWN;
			
				if (!collisionH(player.x[0], player.y[0] + 7 + sy))
				{
					player.y[0] += sy;
				}
				if (!collisionH(player.x[1], player.y[1] + 7 + sy))
				{
					player.y[1] += sy;
				}
			}
			if (cpct_isKeyPressed(Key_CursorRight))
			{
	        	player.frameSet = tileset_masfly_right;	
        		player.framesInSet = PLAYER_HORIZ_ANIM;
        		player.spr = player.frameSet[player.frameCounter % player.framesInSet];
        		player.frameCounter++;
        		player.orientation = RIGHT;

	            if (!collision(player.x[0] + 7 + sx, player.y[0]    ) &&
	            	!collision(player.x[0] + 7 + sx, player.y[0] + 2) &&
	            	!collision(player.x[0] + 7 + sx, player.y[0] + 4) &&
	            	!collision(player.x[0] + 7 + sx, player.y[0] + 6))
	            {
	            	player.x[0] += sx;
	            }
	            if (
	            	!collision(player.x[1] + 7 + sx, player.y[1]) &&
	            	!collision(player.x[1] + 7 + sx, player.y[1] + 2) &&
	            	!collision(player.x[1] + 7 + sx, player.y[1] + 4) &&
	            	!collision(player.x[1] + 7 + sx, player.y[1] + 6))
	            {
	            	player.x[1] += sx;
	            }
	        }
			else if (cpct_isKeyPressed(Key_CursorLeft))
			{
				player.frameSet = tileset_masfly_left;
				player.framesInSet = PLAYER_HORIZ_ANIM;
				player.spr = player.frameSet[player.frameCounter % player.framesInSet];
				player.frameCounter++;
				player.orientation = LEFT;
				
				if (!collision(player.x[0] - sx, player.y[0]) &&
					!collision(player.x[0] - sx, player.y[0] + 2) &&
					!collision(player.x[0] - sx, player.y[0] + 4) &&
					!collision(player.x[0] - sx, player.y[0] + 6))
				{
					player.x[0] -= sx;
				}
				if (!collision(player.x[1] - sx, player.y[1]) &&
					!collision(player.x[1] - sx, player.y[1] + 2) &&
					!collision(player.x[1] - sx, player.y[1] + 4) &&
					!collision(player.x[1] - sx, player.y[1] + 6))
				{
					player.x[1] -= sx;
				}
			}

			if (cpct_isKeyPressed(Key_Control))
			{
				toggleGun();
			}
			
			if (cpct_isKeyPressed(Key_Space))
			{
				if (bullet1.map == 2)
				{
					createBullet(&bullet1, player.orientation, gunDimension,
						player.x[gunDimension], player.y[gunDimension], &player.timeShot);
				}
				else if (bullet2.map == 2)
				{
					createBullet(&bullet2, player.orientation, gunDimension,
						player.x[gunDimension], player.y[gunDimension], &player.timeShot);
				}
			}

			if (cpct_isKeyPressed(Key_7))
			{
				// Easter Egg
				u8* pvideo = cpct_getScreenPtr(SCR_VMEM, 6 * 8 / 2, 10 * 8);
				cpct_drawSolidBox(pvideo, 0xf0, 6 * TILE_WIDTH / 2, TILE_HEIGHT);
			}


			if(cpct_isKeyPressed(Key_C))
			{
				if (!plutonium.catched &&
					abs(plutonium.x - player.x[plutonium.map]) <= 8  &&
					abs(plutonium.y - player.y[plutonium.map]) <= 8)
				{
					UraAux = 1;
					if (UraTime == TIME_PLUTONIUM)
					{ 
						plutonium.catched=1;
						catchedPlutonium();
						UraAux = 0;							
						UraTime = 0;
					}
				}

				if (plutonium.catched && 
					abs(flux.x - player.x[flux.map]) <= 8  &&
					abs(flux.y - player.y[flux.map]) <= 8)
				{
					UraAux = 1;
					if (UraTime == TIME_PLUTONIUM)
					{
						initPlutonium();

						if (++flux.plutonium == flux.fullPlutonium)
						{
							// Yes, you got it!
							gameOver = 1;
						}
						else
						{
							// Añadimos unos pocos enemigos más ;)
							initRandomEnemies(level / 4);
						}

						player.score += PLUTONIUM_SCORE + PLUTONIUM_EXTRA_SCORE * flux.plutonium;
					}
				}

				if (!medpack.catched &&
					abs(medpack.x - player.x[medpack.map]) <= 8  &&
					abs(medpack.y - player.y[medpack.map]) <= 8)
				{
					UraAux = 1;
					if (UraTime == TIME_PLUTONIUM)
					{ 
						medpack.catched=1;
						catchedMedPack();
						UraAux = 0;							
						UraTime = 0;
					}
				}
			}
			else
			{
				UraAux = 0;
			}

			cpct_drawSolidBox(player.players[0], 0xf0, TILE_WIDTH / 2, TILE_HEIGHT);
			player.players[0] = cpct_getScreenPtr(SCR_VMEM, player.x[0] / 2, player.y[0]);
			cpct_drawSprite((void*) player.spr, player.players[0], TILE_WIDTH / 2, TILE_HEIGHT);

			cpct_drawSolidBox(player.players[1], 0xf0, TILE_WIDTH / 2, TILE_HEIGHT);
			player.players[1] = cpct_getScreenPtr(SCR_VMEM, player.x[1] / 2, player.y[1]);
			cpct_drawSprite((void*) player.spr, player.players[1], TILE_WIDTH / 2, TILE_HEIGHT);

			if (player.life <= 0)
				gameOver = -1;
		}
		else // Even frames
		{
			if (bullet1.map != 2)
			{
				updateBullet(&bullet1);
			}
			if (bullet2.map != 2)
			{
				updateBullet(&bullet2);
			}

			if (enemies[0].type != TYPE_NONE)
				enemies[0].update(&enemies[0]);
			if (enemies[1].type != TYPE_NONE)
				enemies[1].update(&enemies[1]);
			if (enemies[2].type != TYPE_NONE)
				enemies[2].update(&enemies[2]);
			if (enemies[3].type != TYPE_NONE)
				enemies[3].update(&enemies[3]);
		}
		
		// FIXME: Debug purpose only, regenerate map.
		if (cpct_isKeyPressed(Key_R))
		{
			return 0;
		}

		if (frame % 50 == 0) // Executed each second.
		{
		    // Draw time.
		    cpct_drawCharM0(TIME_NUM_POS, 1, 5, (levelTime % 10) + '0');
		    if (frame % 500 == 0) // Each 10 s
		    	cpct_drawCharM0(TIME_NUM_POS - 4, 1, 5, ((levelTime / 10) % 10) + '0');
		    if (frame % 5000 == 0) // Each 100 s
		    	cpct_drawCharM0(TIME_NUM_POS - 8, 1, 5, ((levelTime / 100) % 10) + '0');
		    if (levelTime-- == 0) // TIME OVER
		    {
		    	gameOver = -1;
		    }

		    //Catch urarium time
		    if(UraAux)
		    {
		    	UraTime++;
		    }
		    else
		    {
		    	UraTime = 0;
		    }
		}

		frame++;
	}

	if (gameOver == -1)
	{
		showGameOver();
		return 0;
	}

	// Los segundos sobrantes se transforman en puntos.
	player.score += levelTime;

	return level + 1;
}


char sceneGame()
{
	frame = 0;
	level = 1;
	player.score = 0;
	player.life = PLAYER_LIFE;

	do
	{
		level = playLevel();

		if (level == 100)
		{
			showYouWin();
			break;
		}
	}
	while (level > 0);

	return SCENE_MENU;
}
