
//  "Ántrax CPC Game": An Amstrad CPC Game
//  Copyright (C) 2015  Ántrax CPC Team
//
//  This file is part of "Ántrax CPC Game".
//
//  "Ántrax CPC Game" is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  "Ántrax CPC Game" is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <cpctelera.h>
#include <stdlib.h>

#include "utils.h"


/*
 *  0: No collision
 *  1: Collides with something that is already painted.
 */
u8 collision(i16 x, i16 y)
{
    u8* pvideo;

    if (x < 0 || y < 0 || x >= SCREEN_WIDTH || y > 168)
    {
        return -1; 
    }

    pvideo = cpct_getScreenPtr(SCR_VMEM, x / 2, y);

    if (*pvideo != 0xf0)
    {
    	return 1;
    }

    return 0;
}

u8 collisionH(i16 x, i16 y)
{
    u8* pvideo;

    if (x < 0 || y < 0 || x >= SCREEN_WIDTH || y > 168)
    {
        return -1; 
    }

    pvideo = cpct_getScreenPtr(SCR_VMEM, x / 2, y);

    if (*pvideo != 0xf0 || *(pvideo + 1) != 0xf0 ||
        *(pvideo + 2) != 0xf0 || *(pvideo + 3) != 0xf0)
    {
        return 1;
    }

    return 0;
}

u16 manhattan(i16 x1, i16 y1, i16 x2, i16 y2)
{
    return abs(x1 - x2) + abs(y1 - y2);
}

u8 manhattanOnlyX(u8 x1, u8 y1, u8 x2, u8 y2)
{
    u8* pvideo = cpct_getScreenPtr(SCR_VMEM, (x1 * 8 + 3) / 2, y1 * 8 + 3);
    u8 i = 0;

    if (x1 != x2)
    {
        while (x1 < x2)
        {
            if (*(pvideo + 4 * i) != 0xf0)
                return INFINITE_U8;
            x1++;
            i++;
        }
        while (x1 > x2)
        {
            if (*(pvideo - 4 * i) != 0xf0)
                return INFINITE_U8;
            x1--;
            i++;
        }
    }

    return i;
}

u8 manhattanOnlyY(u8 x1, u8 y1, u8 x2, u8 y2)
{
    u8* pvideo = cpct_getScreenPtr(SCR_VMEM, (x1 * 8 + 3) / 2, y1 * 8 + 3);
    u8 i = 0;

    if (y1 != y2)
    {
        while (y1 < y2)
        {
            if ((*(pvideo + 80 * i) != 0xf0))
                return INFINITE_U8;
            y1++;
            i++;
        }
        while (y1 > y2)
        {
            if ((*(pvideo - 80 * i) != 0xf0))
                return INFINITE_U8;
            y1--;
            i++;
        }
    }

    return i;
}

u8 manhattanY(u8 x1, u8 y1, u8 x2, u8 y2)
{
    u8* pvideo;
    u8 i = 0;

    pvideo = cpct_getScreenPtr(SCR_VMEM, (x1 * 8 + 3) / 2, y1 * 8 + 3);

    if (y1 != y2)
    {
        while (y1 < y2)
        {
            if ((*(pvideo + 80 * i) != 0xf0))
                return INFINITE_U8;
            y1++;
            i++;
        }
        while (y1 > y2)
        {
            if ((*(pvideo - 80 * i) != 0xf0))
                return INFINITE_U8;
            y1--;
            i++;
        }
    }

    return i + manhattanOnlyX(x1, y2, x2, y2);
}

u8 manhattanX(u8 x1, u8 y1, u8 x2, u8 y2)
{
    u8* pvideo = cpct_getScreenPtr(SCR_VMEM, (x1 * 8 + 3) / 2, y1 * 8 + 3);
    u8 i = 0;

    if (x1 != x2)
    {
        while (x1 < x2)
        {
            if ((*(pvideo + 4 * i) != 0xf0))
                return INFINITE_U8;
            x1++;
            i++;
        }
        while (x1 > x2)
        {
            if ((*(pvideo - 4 * i) != 0xf0))
                return INFINITE_U8;
            x1--;
            i++;
        }
    }

    return i + manhattanOnlyY(x2, y1, x2, y2);
}

u8 manhattanXY(u8 x1, u8 y1, u8 x2, u8 y2)
{
    u8 min1 = manhattanX(x1, y1, x2, y2);
    u8 min2 = manhattanY(x1, y1, x2, y2);

    return (min1 < min2) ? min1 : min2;
}

void releaseAndPushKey()
{
    // Espera a que despulse tecla
    do
    {
        cpct_scanKeyboard_f();
    }
    while (cpct_isAnyKeyPressed_f());

    // Espera a que se pulse otra tecla.
    do
    {
        cpct_scanKeyboard_f();
    }
    while (!cpct_isAnyKeyPressed_f());
}
